/************************************************************************/
/*																		*/
/*	ADCI2C.cpp		--		Definition for ADCI2C library 	    		*/
/*																		*/
/************************************************************************/
/*	Author:		Cristian Fatu											*/
/*	Copyright 2012, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*		This file defines functions for ADCI2C							*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	01/10/2012(CristianF): created										*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include "ADCI2C.h"
#include <Wire.h>


/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ */



/* ------------------------------------------------------------ */
/*        ADCI2C::WriteByteI2C
**
**        Synopsis:
**				WriteByteI2C(bConfig);
**        Parameters:
**				uint8_t bVal - the byte to be written
**        Errors:
**			none
**
**        Description:
**			This function writes a byte to I2C. 
**
**
*/
void ADCI2C::WriteByteI2C(uint8_t bVal)
{
    Wire.beginTransmission(ADCI2C_I2C_ADDR); //start transmission to device 
    Wire.send(bVal);        // send byte
	Wire.endTransmission(); //end transmission	
}

/* ------------------------------------------------------------ */
/*        ACL::GetIntegerValuesNo
**
**        Synopsis:
**				GetIntegerValuesNo(4, bConfig, wVal0, wVal1, wVal2, wVal3);
**        Parameters:
**				- uint8_t bCntChannels  the number of channels to be read - it is important that this number and bConfig are correlated
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
**				- uint16_t *rgwValues - the array where 16 bit values will be read
**				- uint16_t &wVal0	- parameter used to output the integer value read from the first channel (order of the channels is based on channel number)
**				- uint16_t &wVal1	- parameter used to output the integer value read from the second channel (order of the channels is based on channel number)
**				- uint16_t &wVal2	- parameter used to output the integer value read from the third channel (order of the channels is based on channel number)
**				- uint16_t &wVal3	- parameter used to output the integer value read from the fourth channel (order of the channels is based on channel number)
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				This function will read the specified number of channels and output their values in the output parameters.
**				The configuration byte bConfig is adjusted so that it selects exactly bCntChannels number of channels. 
**				If more channels are selected, only the first bCntChannels are kept selected. If less channels are selected, then the first bCntChannels are selected.
**				The order of the channels is considered using channel number, regardless of the order of the channels in the logical expression of bConfig parameter.
**
**
**
*/
void ADCI2C::GetIntegerValuesNo(uint8_t bCntChannels, uint8_t bConfig, uint16_t &wVal0, uint16_t &wVal1, uint16_t &wVal2, uint16_t &wVal3)
{
	// 1. Adjust bConfig, so that it selects exactly bCntChannels channels. 
	// 		1.1. If more are selected, keep only first bCntChannels
	uint8_t bCntSelectedChannels = 0;
	uint16_t wMskChannel;
	for(wMskChannel = ADCI2C_MSK_CH0; wMskChannel <= ADCI2C_MSK_CH3; wMskChannel <<= 1)
	{
		if(bConfig & (uint8_t)wMskChannel)
		{
			if(bCntSelectedChannels >= bCntChannels)
			{
				// the numbers of selected channels exceeds the requested bCntChannels, de-select this channel
				bConfig &= ~(uint8_t)wMskChannel; 
			}
			else
			{	
				bCntSelectedChannels++;
			}
		}
	}
	// 		1.2. If less channels are selected, select the first bCntChannels
	if(bCntSelectedChannels != bCntChannels)
	{
		// force the selected channels, select the first bCntChannels
		bConfig |= ((1 << bCntChannels) - 1) << 4;
	}

	// 2. Read converter values, according to the channels selected in bConfig
	uint16_t rgwVals[4]; // space for storing all the channels values, but only first bCntChannels will be filled
	RawReadChannels(bConfig, bCntChannels, rgwVals);

	// 3. Fill output parameters
	wVal0 = rgwVals[0] & (uint16_t)ADCI2C_MAX_VAL;	// mask out the rest of the bits, others than the 12 data bits
	wVal1 = rgwVals[1] & (uint16_t)ADCI2C_MAX_VAL;	// mask out the rest of the bits, others than the 12 data bits
	wVal2 = rgwVals[2] & (uint16_t)ADCI2C_MAX_VAL;	// mask out the rest of the bits, others than the 12 data bits
	wVal3 = rgwVals[3] & (uint16_t)ADCI2C_MAX_VAL;	// mask out the rest of the bits, others than the 12 data bits
}

/* ------------------------------------------------------------ */
/*        ACL::RawReadChannels
**
**        Synopsis:
**				RawReadChannels(bConfig, bCntBytes, rgbValues);
**        Parameters:
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation) 
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
**				uint8_t bCntChannels   the number of channels to be converted. The value has to correspond to the channels specified in the bConfig parameter.
**				uint16_t *rgwValues - the array where the 16-bit value/s will be placed . It must be allocated according to bCntChannels value.
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				This function will read from I2C the values of the specified channels and store the received values in the buffer. Each value contains channel number on bits 13, 12 (see AD7991 documentation).
**				The order in which the channels values are stored is set according to the channel number, e.g. when using bConfig = ADCI2C_MSK_CH3 | ADCI2C_MSK_CH1, it will store channel 1 value first, then channel 3.
**				When calling the function, it is important that bCntChannels contains the number of channels selected in bConfig.
**				This is the lowest level function that can be used to read channels values, and it offers the best performance.
**
**
*/
void ADCI2C::RawReadChannels(uint8_t bConfig, uint8_t bCntChannels, uint16_t *rgwValues)
{              
	// 1. Write Configuration Register
	WriteByteI2C(bConfig);	uint8_t *rgbValues = (uint8_t *)rgwValues;
	// 2. Read converter values, according to the channels selected in bConfig
	uint8_t bCntBytes = bCntChannels << 1;
	Wire.beginTransmission(ADCI2C_I2C_ADDR); 	//start transmission to idDevice (initiate again)
	Wire.requestFrom(ADCI2C_I2C_ADDR, (int)bCntBytes);    // request bCntBytes bytes from idDevice
                	
	int nIdxBytes = 0;
	
	while(Wire.available() && nIdxBytes <= bCntBytes - 1)    //idDevice might send less than requested (abnormal)
	{ 
		rgbValues[nIdxBytes ^ 1] = Wire.receive();// receive a byte
		nIdxBytes++;
	}
	
	Wire.endTransmission(); //end transmission
}


/* ------------------------------------------------------------ */
/*        ADCI2C::GetIntegerValue1Ch
**
**        Synopsis:
**				GetIntegerValue1Ch(wVal0);
**        Parameters:
**				- uint16_t &wVal0	- parameter used to output the integer value read from the channel 
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						It can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
						Default value: channel 0 is selected (ADCI2C_MSK_CH0)
**
**        Return Values:
**                none
**
**        Errors:
**			none
**
**        Description:
**			This function converts one analog input channel according to the configuration byte. The user may select any channel, using bConfig parameter.
**			The configuration byte bConfig is set accordingly, so that it selects exactly one channel.  
**			If more channels are selected, only the first channel is maintained selected. If no channels are selected, then the Channel 0 is selected.
**			If bConfig parameter is not provided, the Channel 0 is selected and no other configuration option is used.
**			If bConfig parameter is provided, along with it the user  selects the channel to be converted.
**
**
*/

void ADCI2C::GetIntegerValue1Ch(uint16_t &wVal0, uint8_t bConfig)
{
	uint16_t wValDummy1, wValDummy2, wValDummy3;
	GetIntegerValuesNo(1, bConfig, wVal0, wValDummy1, wValDummy2, wValDummy3);
}

/* ------------------------------------------------------------ */
/*        ADCI2C::GetIntegerValues2Ch
**
**        Synopsis:
**				GetIntegerValues2Ch(wVal0, wVal1);
**        Parameters:
**				- uint16_t &wVal0	- parameter used to output the integer value read from the first channel (order of the channels is based on the channel number)
**				- uint16_t &wVal1	- parameter used to output the integer value read from the second channel (order of the channels is based on the channel number)
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
						Default value: channels 0 and 1 are selected (ADCI2C_MSK_CH0 | ADCI2C_MSK_CH1)
**
**        Return Values:
**                none
**
**        Errors:
**			none
**
**        Description:
**			This function converts two analog input channels according to the configuration byte. The user may select any combination of two channels, using the bConfig parameter.
**			The configuration byte bConfig is set accordingly so that it selects exactly two channels. 
**			If more channels are selected, only the first two channels are maintained selected. If less than two channels are selected, then the first two channels (0 and 1) are selected by default.
**			If bConfig parameter is not provided, the first two channels are selected and no other configuration option is used.
**			If bConfig parameter is provided, the user must select the channels to be converted.
**			The order of the channels is set using the channel number, regardless of the order of the channels in the logical expression of bConfig parameter.
**
**
*/
void ADCI2C::GetIntegerValues2Ch(uint16_t &wVal0, uint16_t &wVal1, uint8_t bConfig)
{
	uint16_t wValDummy2, wValDummy3;
	GetIntegerValuesNo(2, bConfig, wVal0, wVal1, wValDummy2, wValDummy3);
}

/* ------------------------------------------------------------ */
/*        ADCI2C::GetIntegerValues3Ch
**
**        Synopsis:
**				GetIntegerValues3Ch(wVal0, wVal1, wVal2);
**        Parameters:
**				- uint16_t &wVal0	- parameter used to output the integer value read from the first channel (order of the channels is based on the channel number)
**				- uint16_t &wVal1	- parameter used to output the integer value read from the second channel (order of the channels is based on the channel number)
**				- uint16_t &wVal2	- parameter used to output the integer value read from the third channel (order of the channels is based on the channel number)
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
						Default value: channels 0, 1 and 2 are selected (ADCI2C_MSK_CH0 | ADCI2C_MSK_CH1 | ADCI2C_MSK_CH2)
**
**        Return Values:
**                none
**
**        Errors:
**			none
**
**        Description:
**			This function converts three analog input channels according to the configuration byte. The user can select any combination of three channels, using bConfig parameter.
**			The configuration byte bConfig is set accordingly so that it selects exactly three channels. 
**			If more channels are selected, only the first three channels are maintained selected. If less than three channels are selected, then the first three channels (0, 1 and 2) are selected.
**			If bConfig parameter is not provided, the first three channels are selected and no other configuration option is used.
**			If bConfig parameter is provided, the user must select the channels to be converted.
**			The order of the channels is set using the channel number, regardless of the order of the channels in the logical expression of bConfig parameter.
**
**
*/
void ADCI2C::GetIntegerValues3Ch(uint16_t &wVal0, uint16_t &wVal1, uint16_t &wVal2, uint8_t bConfig)
{
	uint16_t wValDummy3;
	GetIntegerValuesNo(3, bConfig, wVal0, wVal1, wVal2, wValDummy3);
}

/* ------------------------------------------------------------ */
/*        ADCI2C::GetIntegerValues4Ch
**
**        Synopsis:
**				GetIntegerValues4Ch(wVal0, wVal1, wVal2, wVal3);
**        Parameters:
**				- uint16_t &wVal0	- parameter used to output the integer value read from the first channel (order of the channels is based on the channel number)
**				- uint16_t &wVal1	- parameter used to output the integer value read from the second channel (order of the channels is based on the channel number)
**				- uint16_t &wVal2	- parameter used to output the integer value read from the third channel (order of the channels is based on the channel number)
**				- uint16_t &wVal3	- parameter used to output the integer value read from the fourth channel (order of the channels is based on the channel number)
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
						Default value: all channels selected (ADCI2C_MSK_CH0 | ADCI2C_MSK_CH1 | ADCI2C_MSK_CH2 | ADCI2C_MSK_CH3)
**
**        Return Values:
**                none
**
**        Errors:
**			none
**
**        Description:
**			This function converts the four analog input channels according to the configuration byte.  
**			The order of the channels is set using channel number, regardless of the order of the channels in the logical expression of bConfig parameter.
**			If less than four channels are selected in bConfig, then all the four channels are selected.
**			If bConfig parameter is not provided, all the four channels are selected and no other configuration option is used.
**
**
*/
void ADCI2C::GetIntegerValues4Ch(uint16_t &wVal0, uint16_t &wVal1, uint16_t &wVal2, uint16_t &wVal3, uint8_t bConfig)
{
	GetIntegerValuesNo(4, bConfig, wVal0, wVal1, wVal2, wVal3);
}

/* ------------------------------------------------------------ */
/*        ADCI2C::GetPhysicalValue1Ch
**
**        Synopsis:
**				GetPhysicalValue1Ch(fVal0);
**        Parameters:
**        Parameters:
**				- float &fVal0	- parameter used to output the physical value corresponding to the channel 
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
						Default value: channel 0 is selected  (ADCI2C_MSK_CH0)
**				- float dReference - the value corresponding to the maximum converted value. 
						Default value: 3.3
**					
**        Return Values:
**                none
**
**        Errors:
**			none
**
**        Description:
**			This function converts one analog input channel according to the configuration byte. The user may select any channel, using bConfig parameter.
**			The configuration byte bConfig is set accordingly so that it selects exactly one channel.   
**			If more channels are selected, only the first channel is maintained selected. If no channels are selected, then the Channel 0 is selected.
**			The function computes the physical value corresponding to the read (integer) value for the selected channel and to the reference value.
**			If bConfig parameter is not provided, the Channel 0 is selected and no other configuration option is used.
**			If bConfig parameter is provided, the user must select the channel to be converted.
**			If dReference parameter is not provided, 3.3 value is used as reference value. 
**
**
*/
void ADCI2C::GetPhysicalValue1Ch(float &fVal0, uint8_t bConfig, float dReference)
{
	uint16_t wVal0, wVal1Dummy, wVal2Dummy, wVal3Dummy;
	GetIntegerValuesNo(1, bConfig, wVal0, wVal1Dummy, wVal2Dummy, wVal3Dummy);
	fVal0 = (float)wVal0 * (dReference / ADCI2C_MAX_VAL);
}


/* ------------------------------------------------------------ */
/*        ADCI2C::GetPhysicalValues2Ch
**
**        Synopsis:
**				GetPhysicalValues2Ch(fVal0, fVal1);
**        Parameters:
**        Parameters:
**				- float &fVal0	- parameter used to output the physical value corresponding to the first channel (order of the channels is based on the channel number)
**				- float &fVal1	- parameter used to output the physical value corresponding to the second channel (order of the channels is based on the channel number)
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion	
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
						Default value: channels 0 and 1 are selected  (ADCI2C_MSK_CH0 | ADCI2C_MSK_CH1)
**				- float dReference - the value corresponding to the maximum converted value. 
						Default value: 3.3
**					
**        Return Values:
**                none
**
**        Errors:
**			none
**
**        Description:
**			This function converts two analog input channels according to the configuration byte. User may select any combination of two channels, using the bConfig parameter.
**			The configuration byte bConfig is set accordingly so that it selects exactly two  channels. 
**			If more channels are selected, only the first two channels are kept selected. If less than two channels are selected, then the first two channels (0 and 1) are selected.
**			For each channel, the function computes the physical values corresponding to the read (integer) values and to the reference value.
**			If bConfig parameter is not provided, the first two channels are selected and no other configuration option is used.
**			If bConfig parameter is provided, the user must select the channels to be converted.
**			The order of the channels is set using the channel number, regardless of the order of the channels in the logical expression of bConfig parameter.
**			If dReference parameter is not provided, 3.3 value is used as reference value.
**
**
*/
void ADCI2C::GetPhysicalValues2Ch(float &fVal0, float &fVal1, uint8_t bConfig, float dReference)
{
	uint16_t wVal0, wVal1, wVal2Dummy, wVal3Dummy;
	GetIntegerValuesNo(2, bConfig, wVal0, wVal1, wVal2Dummy, wVal3Dummy);
	fVal0 = (float)wVal0 * (dReference / ADCI2C_MAX_VAL);
	fVal1 = (float)wVal1 * (dReference / ADCI2C_MAX_VAL);
}

/* ------------------------------------------------------------ */
/*        ADCI2C::GetPhysicalValues3Ch
**
**        Synopsis:
**				GetPhysicalValues3Ch(fVal0, fVal1, fVal2);
**        Parameters:
**        Parameters:
**				- float &fVal0	- parameter used to output the physical value corresponding to the first channel (order of the channels is based on the channel number)
**				- float &fVal1	- parameter used to output the physical value corresponding to the second channel (order of the channels is based on the channel number)
**				- float &fVal2	- parameter used to output the physical value corresponding to the third channel (order of the channels is based on the channel number)
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
						Default value: channels 0, 1 and 2 are selected (ADCI2C_MSK_CH0 | ADCI2C_MSK_CH1 | ADCI2C_MSK_CH2)
**				- float dReference - the value corresponding to the maximum converted value. 
						Default value: 3.3
**					
**        Return Values:
**                none
**
**        Errors:
**			none
**
**        Description:
**			This function converts three analog input channels according to the configuration byte. The user may select any combination of three channels, using bConfig parameter.
**			The configuration byte bConfig is adjusted so that it selects exactly three channels. 
**			If more channels are selected, only the first three are maintained selected. If less than three channels are selected, then the first three channels (0, 1 and 2) are selected.
**			For each channel, the function computes the physical values corresponding to the read (integer) values and to the reference value.
**			If bConfig parameter is not provided, the first three channels are selected and no other configuration option is used.
**			If bConfig parameter is provided, the user must select the channels to be converted.
**			If dReference parameter is not provided, 3.3 value is used as reference value.
**
**
*/
void ADCI2C::GetPhysicalValues3Ch(float &fVal0, float &fVal1, float &fVal2, uint8_t bConfig, float dReference)
{
	uint16_t wVal0, wVal1, wVal2, wVal3Dummy;
	GetIntegerValuesNo(3, bConfig, wVal0, wVal1, wVal2, wVal3Dummy);
	fVal0 = (float)wVal0 * (dReference / ADCI2C_MAX_VAL);
	fVal1 = (float)wVal1 * (dReference / ADCI2C_MAX_VAL);
	fVal2 = (float)wVal2 * (dReference / ADCI2C_MAX_VAL);
}

/* ------------------------------------------------------------ */
/*        ADCI2C::GetPhysicalValues4Ch
**
**        Synopsis:
**				GetPhysicalValues4Ch(fVal0, fVal1, fVal2, fVal3);
**        Parameters:
**				- float &fVal0	- parameter used to output the physical value corresponding to the first channel (order of the channels is based on the channel number)
**				- float &fVal1	- parameter used to output the physical value corresponding to the second channel (order of the channels is based on the channel number)
**				- float &fVal2	- parameter used to output the physical value corresponding to the third channel (order of the channels is based on the channel number)
**				- float &fVal3	- parameter used to output the physical value corresponding to the fourth channel (order of the channels is based on the channel number)
**				- uint8_t bConfig	- the configuration settings, specifying which channels are selected for conversion and other options
						it can be one or more (OR-ed) options from the list:
							ADCI2C_MSK_CH0				- 	channel 0 is selected for conversion
                            ADCI2C_MSK_CH1				- 	channel 1 is selected for conversion 	
				            ADCI2C_MSK_CH2				- 	channel 2 is selected for conversion
                            ADCI2C_MSK_CH3				- 	channel 3 is selected for conversion
				            ADCI2C_MSK_REF_SEL			- 	if this bit is set to 1, an external reference must be used and applied to the VIN3/VREF pin of the module
                            ADCI2C_MSK_FLTR				- 	filtering on SDA and SCL lines is enabled (0) or bypassed (1).
                            ADCI2C_MSK_BIT_TRIAL_DELAY	- 	value 1 to turn off bit trial delay (see AD7991 documentation)
                            ADCI2C_MSK_SAMPLE_DELAY		- 	value 1 to turn off sample delay (see AD7991 documentation)
						Default value: all channels selected (ADCI2C_MSK_CH0 | ADCI2C_MSK_CH1 | ADCI2C_MSK_CH2 | ADCI2C_MSK_CH3)
**				- float dReference - the value corresponding to the maximum converted value. 
						Default value: 3.3
**					
**        Return Values:
**                none
**
**        Errors:
**			none
**
**        Description:
**			This function converts the four analog input channels according to the configuration byte.  
**			For each channel, the function computes the physical values corresponding to the read (integer) values and to the reference value.
**			The order of the channels is set using channel number, regardless of the order of the channels in the logical expression of bConfig parameter.
**			If less than four channels are selected in bConfig, then all four are selected.
**			If bConfig parameter is not provided, all the four channels are selected and no other configuration option is used. 
**			If dReference parameter is not provided, 3.3 value is used as reference value.
**
**
*/
void ADCI2C::GetPhysicalValues4Ch(float &fVal0, float &fVal1, float &fVal2, float &fVal3, uint8_t bConfig, float dReference)
{
	uint16_t wVal0, wVal1, wVal2, wVal3;
	GetIntegerValues4Ch(wVal0, wVal1, wVal2, wVal3, bConfig);
	fVal0 = (float)wVal0 * (dReference / ADCI2C_MAX_VAL);
	fVal1 = (float)wVal1 * (dReference / ADCI2C_MAX_VAL);
	fVal2 = (float)wVal2 * (dReference / ADCI2C_MAX_VAL);
	fVal3 = (float)wVal3 * (dReference / ADCI2C_MAX_VAL);
}


/* ------------------------------------------------------------ */
/*        ADCI2C::begin
**
**        Synopsis:
**				myADCI2C.begin();
**        Parameters:
**
**
**        Return Values:
**                void 
**
**        Errors:
**
**
**        Description:
**				This function initializes the I2C interface #1 that is used to communicate with PmodAD2.
**
**
*/
void ADCI2C::begin()
{
	Wire.begin();
}





